/*
 * DANG_BEGIN_MODULE
 * 
 * PHASE1 initialization:
 *
 * This is where you initialize anything necessary to create a virtual
 * PC configuration.  Intialize the cpu, the various chips (pic, pit, etc.)
 * and then setup & initialize the BIOS, etc.
 * 
 * DANG_END_MODULE
 * 
 * DANG_BEGIN_CHANGELOG
 *
 * $Date: $
 * $Source: $
 * $Revision: $
 * $State: $
 *
 * $Log: $
 *
 * DANG_END_CHANGELOG
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "dosemu.h"
#include "emu_mem.h"
#include "emu_bios.h"
#include "emu_mouse.h"
#include "emu_iodev.h"
#include "bios_entry.h"

/* 
 * DANG_BEGIN_FUNCTION device_init
 * 
 * description:
 *  Calls all initialization routines for devices (keyboard, video, serial,
 *    disks, etc.)
 *
 * DANG_END_FUNCTION
 */
static inline void device_init(void)
{
  mouse_init();
}

static inline void device_reset(void)
{
  keyboard_flags_init();
  video_config_reset();
  mouse_close();
  mouse_init();
}

/* 
 * DANG_BEGIN_FUNCTION bios_mem_setup
 * 
 * description:
 *  Set up all memory areas as would be present on a typical i86 during
 * the boot phase.
 *
 * DANG_END_FUNCTION
 */
static inline void bios_mem_setup(void)
{
  int b;

  keybuf_clear();

#if 0
  bios_ctrl_alt_del_flag = 0x0000;
  bios_keyboard_flags2   = 16;	/* 102-key keyboard */
#endif

  WRITE_WORD(LASTSCAN_ADD, 1);
}

/*
 * DANG_BEGIN_FUNCTION phase1_init
 * 
 * description:
 *  Fire up phase 1
 *
 * DANG_END_FUNCTION
 */
void phase1_init(void)
{
  emulator_init();

  /* Memory init */
  a20_init();

  /* Virtual hardware init */
  cpu_init();		        /* initialize the CPU */
  iodev_init();                 /* initialize the io-devices */
  device_init();		/* initialize keyboard, disk, video, etc. */

  /* BIOS setup */
  bios_init();                  /* initialize the bios */

  env_init_phase1();

  /* Shared memory init */
  shared_memory_init();
  shared_keyboard_init();
}

void phase1_reset(void)
{
  static Boolean first_reset = TRUE;

  emulator_reset();

  /* Memory init */
  a20_reset();

  /* Virtual hardware init */
  cpu_init();
  iodev_reset();
  device_reset();

  /* BIOS setup */
  bios_init();                  /* initialize the bios */
  bios_mem_setup();

  env_reset_phase1();

  g_printf("BIOS: dosemu_banner, old CS:IP = 0x%04x:0x%04x\n", _CS, _IP);
  _CS = BIOSSEG;
  _IP = ROM_BIOS_SELFTEST;

  if (!first_reset) {
    clear_screen(READ_BYTE(BIOS_CURRENT_SCREEN_PAGE), 7);
    dbug_printf("DOS ctrl-alt-del requested.  Rebooting!\n");
    p_dos_str("Rebooting DOS.  Be careful, this is partially implemented\r\n");
  }
  else
    first_reset = FALSE;
}

