/* 
 * DANG_BEGIN_MODULE
 * 
 * Contains the run_vm86 function
 *
 * DANG_END_MODULE
 *
 * DANG_BEGIN_CHANGELOG
 *
 * $Date: $
 * $Source: $
 * $Revision: $
 * $State: $
 *
 * $Log: $
 *
 * DANG_END_CHANGELOG
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "dosemu.h"
#include "emu_mem.h"
#include "emu_cpu.h"
#include "emu_port.h"
#include "emu_int.h"
#include "emu_pic.h"
#include "emu_port.h"
#include "emu_hlt.h"
#include "emu_timers.h"

/*
 * DANG_BEGIN_FUNCTION vm86_GP_fault();
 *
 * description:
 * All from the kernel unhandled general protection faults from V86 mode
 * are handled here. This are mainly port IO and the HLT instruction.
 *
 * DANG_END_FUNCTION
 */

void vm86_GP_fault(void)
{
  Bit8u *csp;

  if (ignore_segv) {
    error("ERROR: sigsegv ignored!\n");
    return;
  }

  if (in_sigsegv)
    error("ERROR: in_sigsegv=%d!\n", in_sigsegv);

  /* in_vm86 = 0; */
  in_sigsegv++;

  /* DANG_BEGIN_REMARK 
   * In a properly functioning emulator :-), sigsegv's will never come
   * while in a non-reentrant system call (ioctl, select, etc).  Therefore,
   * there's really no reason to worry about them, so I say that I'm NOT
   * in a signal handler (I might make this a little clearer later, to
   * show that the purpose of in_sighandler is to stop non-reentrant system
   * calls from being reentered.
   * I reiterate: sigsegv's should only happen when I'm running the vm86
   * system call, so I really shouldn't be in a non-reentrant system call
   * (except maybe vm86)
   * - Robert Sanders
   * DANG_END_REMARK
   */
  in_sighandler = 0;

  if (LWORD(eflags) & TF) {
    g_printf("SIGSEGV received while TF is set\n");
    show_regs(__FILE__, __LINE__);
  }

  csp = SEG_ADR((Bit8u *), cs, ip);

  switch (*csp) {
  case 0x6c:                    /* insb */
    *(SEG_ADR((Bit8u *),es,di)) = port_inb(_DX);
    i_printf("insb(0x%04x) value %02x\n", _DX, *(SEG_ADR((Bit8u *),es,di)));
    if (LWORD(eflags) & DF)
      _DI--;
    else
      _DI++;
    _IP++;
    break;

  case 0x6d:			/* insw */
    *(SEG_ADR((Bit16u *),es,di)) = port_inw(_DX);
    i_printf("insw(0x%04x) value %04x\n", _DX, *(SEG_ADR((Bit16u *),es,di)));
    if (LWORD(eflags) & DF)
      _DI -= 2;
    else
      _DI +=2;
    _IP++;
    break;
      
  case 0x6e:			/* outsb */
    i_printf("outsb(0x%04x) value %02x\n", _DX, *(SEG_ADR((Bit8u *),ds,si)));
    port_outb(_DX, *(SEG_ADR((Bit8u *),ds,si)));
    if (LWORD(eflags) & DF)
      _SI--;
    else
      _SI++;
    _IP++;
    break;

  case 0x6f:			/* outsw */
    i_printf("outsw(0x%04x) value %04x\n", _DX, *(SEG_ADR((Bit16u *),ds,si)));
    port_outw(_DX, *(SEG_ADR((Bit16u *),ds,si)));
    if (LWORD(eflags) & DF)
      _SI -= 2;
    else
      _SI += 2;
    _IP++;
    break;

  case 0xe5:			/* inw xx */
    _AX = port_inw(csp[1]);
    _IP += 2;
    break;
  case 0xe4:			/* inb xx */
    _AL = port_inb(csp[1]);
    _IP += 2;
    break;

  case 0xed:			/* inw dx */
    _AX = port_inw(_DX);
    _IP++;
    break;
  case 0xec:			/* inb dx */
    _AL = port_inb(_DX);
    _IP++;
    break;

  case 0xe7:			/* outw xx */
    port_outw(csp[1], _AX);
    _IP += 2;
    break;
  case 0xe6:			/* outb xx */
    port_outb(csp[1], _AL);
    _IP += 2;
    break;

  case 0xef:			/* outw dx */
    port_outw(_DX, _AX);
    _IP++;
    break;
  case 0xee:			/* outb dx */
    port_outb(_DX, _AL);
    _IP++;
    break;

  case 0xf3:                    /* rep */
    switch(csp[1]) {                
      case 0x6c: {             /* rep insb */
        int delta = 1;
        if (LWORD(eflags) & DF) delta = -1;
        i_printf("Doing REP F3 6C (rep insb) %04x bytes, DELTA %d\n",
                 _CX, delta);
        for ( ; _CX; _CX--, _DI += delta) {
          *(SEG_ADR((Bit8u *),es,di)) = port_inb(_DX);
        }
        _IP += 2;
        break;
      }
      case  0x6d: {           /* rep insw */
        int delta = 2;
        if (LWORD(eflags) & DF) delta = -2;
        i_printf("REP F3 6D (rep insw) %04x words, DELTA %d\n", _CX, delta);
        for ( ; _CX; _CX--, _DI += delta) {
          *(SEG_ADR((Bit16u *),es,di)) = port_inw(_DX);
	}
	_IP += 2;
	break;
      }
      case 0x6e: {           /* rep outsb */
        int delta = 1;
        i_printf("untested: rep outsb\n");
        if (LWORD(eflags) & DF) delta = -1;
        for ( ; _CX; _CX--, _DI += delta) {
          port_outb(_DX, *(SEG_ADR((Bit8u *),ds,si)));
        }
        _IP += 2;
        break;
      }
      case 0x6f: { 
        int delta = 2;
        i_printf("untested: rep outsw\n");
        if (LWORD(eflags) & DF) delta = -2;
        for ( ; _CX; _CX--, _DI += delta) {
          port_outw(_DX, *(SEG_ADR((Bit16u *),ds,si)));
        }
        _IP += 2;
        break;
      }
      default:
	i_printf("Nope REP F3, CSP[1] = 0x%04x\n", csp[1]);
    }
    break;        

  case 0xf4:
    hlt_handle();
    break;

  case 0xf0:			/* lock */
  default:
    /* er, why don't we advance eip here, and why does it work??  */
    error("general protection at %p: %x\n", csp, *csp);
    show_regs(__FILE__, __LINE__);
    show_ints(0, 0x33);
    error("ERROR: SIGSEGV, protected insn...exiting!\n");
    fatalerr = 4;
    leavedos(fatalerr);		/* shouldn't return */
  }				/* end of switch() */

  if (LWORD(eflags) & TF) {
    g_printf("TF: trap done");
    show_regs(__FILE__, __LINE__);
  }

  in_sigsegv--;
  in_sighandler = 0;
}

/*
 * DANG_BEGIN_FUNCTION run_vm86
 * 
 * description: Here is where DOSEMU runs VM86 mode with the vm86() call
 * which also has the registers that it will be called with.  It will stop
 * vm86 mode for many reasons, like trying to execute an interrupt, doing
 * port I/O to ports not opened for I/O, etc ...
 * 
 * DANG_END_FUNCTION
 */
void run_vm86(void)
{
  int retval;
  
#ifdef CONFIG_DEBUG_VERBOSE
  g_printf("VM86: run_vm86() being called\n");
#if 0
  show_regs(__FILE__, __LINE__);
#endif
#endif
  /*
   * always invoke vm86() with this call.  all the messy stuff will be
   * in here.
   */
  in_vm86 = 1;
  if (pic_icount)
    REG(eflags) |= (VIP);
  /* FIXME: this needs to be clarified and rewritten */

  retval = vm86(&vm86s);
  in_vm86 = 0;
#ifdef CONFIG_DEBUG_VERBOSE
  g_printf("VM86: vm86 returned w/ret = 0x%04x\n", retval);
#if 0
  show_regs(__FILE__, __LINE__);
#endif
#endif
  switch (VM86_TYPE(retval)) {
    case VM86_UNKNOWN:
      vm86_GP_fault();
      break;
    case VM86_STI:
      I_printf("VM86: STI\n");
      pic_iret();
      break;
    case VM86_INTx:
      int_run(VM86_ARG(retval));
      break;
    case VM86_SIGNAL:
      I_printf("VM86: SIGNAL\n");
      break;
    default:
      error("unknown return value from vm86()=%x,%d-%x\n", VM86_TYPE(retval), VM86_TYPE(retval), VM86_ARG(retval));
      fatalerr = 4;
    }

#ifdef CONFIG_DEBUG_VERBOSE
  show_regs(__FILE__, __LINE__);
#endif

  /* update the pic to reflect IEF */
  if (REG(eflags) & IF_MASK) {
    if (pic_iflag)
      pic_sti();
  } else {
    if (!pic_iflag)
      pic_cli();		/* pic_iflag=0 => enabled */
  }

  {
    struct timeval cur_time;
    gettimeofday(&cur_time, NULL);

#ifdef CONFIG_DEBUG_VERBOSE
  g_printf("VM86: calling signal_run()\n");
#endif

  signal_run();

#ifdef CONFIG_DEBUG_VERBOSE
  g_printf("VM86: calling timer_schedule()\n");
#endif

    timer_schedule(&cur_time);
  }

#ifdef CONFIG_DEBUG_VERBOSE
  g_printf("VM86: looking at queued io & pic flags\n");
#endif

  /*
   * This is here because ioctl() is non-reentrant, and signal handlers
   * may have to use ioctl().  This results in a possible (probable)
   * time lag of indeterminate length (and a bad return value). Ah, life
   * isn't perfect.
   * 
   * I really need to clean up the queue functions to use real queues.
   */
  if (iq.queued)
    do_queued_ioctl();
}
